package fr.aminelouveaau.cartes;

import fr.aminelouveaau.cartes.model.Card;
import fr.aminelouveaau.cartes.service.GameService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.util.ReflectionTestUtils;
import org.springframework.web.client.HttpClientErrorException;

import java.util.List;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

@SpringBootTest
class CartesApplicationTests {

    public static final int CARDS_PER_HAND = 10;
    public static final int DECK_SIZE = 4 * 13;

    @InjectMocks
    GameService gameService;

    @BeforeEach
    public void setUp() {
        ReflectionTestUtils.setField(gameService, "cardHandNumber", CARDS_PER_HAND);
        gameService.init();
    }

    @Test
    void testDeckGeneration() {
        List<Card> deck = gameService.getDeck();
        assertThat(deck.size()).isNotEqualTo(0).withFailMessage("Deck is empty");
        assertThat(deck.size()).isEqualTo(DECK_SIZE).withFailMessage("Deck is not complete");
    }

    @Test
    void testHandGeneration() {
        List<Card> hand = gameService.cardHand();
        assertThat(hand.size()).isEqualTo(CARDS_PER_HAND).withFailMessage("Hand is not complete");
        List<Card> deck = gameService.getDeck();
        assertThat(deck.size()).isNotEqualTo(DECK_SIZE).withFailMessage("Where did the cards come from ?");
        assertThat(deck.size()).isEqualTo(DECK_SIZE - CARDS_PER_HAND).withFailMessage("Wrong number of cards removed.");
    }

    @Test()
    void testExceptionTooManyPlayers() {
        int maxNumberOfPlayers = (int) Math.floor(DECK_SIZE / CARDS_PER_HAND);
        for (int i = 0; i < maxNumberOfPlayers; i++) {
            gameService.cardHand();
        }
        Assertions.assertThrows(HttpClientErrorException.class, () -> gameService.cardHand());
    }

    @Test
    void contextLoads() {
    }

}
